@@ -49,6 +49,8 @@ enum fd_param_id {
 	FD_DEVICE_ID,
 	FD_GMEM_SIZE,
 	FD_GPU_ID,
+	FD_CHIP_ID,
+	FD_MAX_FREQ,
 };
 
 /* bo flags: */
@@ -75,6 +77,7 @@ struct fd_device * fd_device_new(int fd);
 struct fd_device * fd_device_new_dup(int fd);
 struct fd_device * fd_device_ref(struct fd_device *dev);
 void fd_device_del(struct fd_device *dev);
+int fd_device_fd(struct fd_device *dev);
 
 
 /* pipe functions:
@@ -85,6 +88,9 @@ void fd_pipe_del(struct fd_pipe *pipe);
 int fd_pipe_get_param(struct fd_pipe *pipe, enum fd_param_id param,
 		uint64_t *value);
 int fd_pipe_wait(struct fd_pipe *pipe, uint32_t timestamp);
+/* timeout in nanosec */
+int fd_pipe_wait_timeout(struct fd_pipe *pipe, uint32_t timestamp,
+		uint64_t timeout);
 
 
 /* buffer-object functions:
@@ -94,11 +100,15 @@ struct fd_bo * fd_bo_new(struct fd_device *dev,
 		uint32_t size, uint32_t flags);
 struct fd_bo * fd_bo_from_fbdev(struct fd_pipe *pipe,
 		int fbfd, uint32_t size);
+struct fd_bo *fd_bo_from_handle(struct fd_device *dev,
+		uint32_t handle, uint32_t size);
 struct fd_bo * fd_bo_from_name(struct fd_device *dev, uint32_t name);
+struct fd_bo * fd_bo_from_dmabuf(struct fd_device *dev, int fd);
 struct fd_bo * fd_bo_ref(struct fd_bo *bo);
 void fd_bo_del(struct fd_bo *bo);
 int fd_bo_get_name(struct fd_bo *bo, uint32_t *name);
 uint32_t fd_bo_handle(struct fd_bo *bo);
+int fd_bo_dmabuf(struct fd_bo *bo);
 uint32_t fd_bo_size(struct fd_bo *bo);
 void * fd_bo_map(struct fd_bo *bo);
 int fd_bo_cpu_prep(struct fd_bo *bo, struct fd_pipe *pipe, uint32_t op);
