@@ -191,7 +191,7 @@ static void kgsl_ringbuffer_destroy(struct fd_ringbuffer *ring)
 	free(kgsl_ring);
 }
 
-static struct fd_ringbuffer_funcs funcs = {
+static const struct fd_ringbuffer_funcs funcs = {
 		.hostptr = kgsl_ringbuffer_hostptr,
 		.flush = kgsl_ringbuffer_flush,
 		.emit_reloc = kgsl_ringbuffer_emit_reloc,
@@ -199,7 +199,7 @@ static struct fd_ringbuffer_funcs funcs = {
 		.destroy = kgsl_ringbuffer_destroy,
 };
 
-struct fd_ringbuffer * kgsl_ringbuffer_new(struct fd_pipe *pipe,
+drm_private struct fd_ringbuffer * kgsl_ringbuffer_new(struct fd_pipe *pipe,
 		uint32_t size)
 {
 	struct kgsl_ringbuffer *kgsl_ring;
