@@ -1,14 +1,16 @@
 #ifndef __NOUVEAU_LIBDRM_PRIVATE_H__
 #define __NOUVEAU_LIBDRM_PRIVATE_H__
 
+#include <libdrm_macros.h>
 #include <xf86drm.h>
 #include <xf86atomic.h>
+#include <pthread.h>
 #include "nouveau_drm.h"
 
 #include "nouveau.h"
 
 #ifdef DEBUG
-uint32_t nouveau_debug;
+drm_private uint32_t nouveau_debug;
 #define dbg_on(lvl) (nouveau_debug & (1 << lvl))
 #define dbg(lvl, fmt, args...) do {                                            \
 	if (dbg_on((lvl)))                                                     \
@@ -94,7 +96,7 @@ nouveau_bo(struct nouveau_bo *bo)
 struct nouveau_device_priv {
 	struct nouveau_device base;
 	int close;
-	atomic_t lock;
+	pthread_mutex_t lock;
 	struct nouveau_list bo_list;
 	uint32_t *client;
 	int nr_client;
@@ -112,13 +114,11 @@ int
 nouveau_device_open_existing(struct nouveau_device **, int, int, drm_context_t);
 
 /* abi16.c */
-int  abi16_chan_nv04(struct nouveau_object *);
-int  abi16_chan_nvc0(struct nouveau_object *);
-int  abi16_chan_nve0(struct nouveau_object *);
-int  abi16_engobj(struct nouveau_object *);
-int  abi16_ntfy(struct nouveau_object *);
-void abi16_bo_info(struct nouveau_bo *, struct drm_nouveau_gem_info *);
-int  abi16_bo_init(struct nouveau_bo *, uint32_t alignment,
-		   union nouveau_bo_config *);
+drm_private bool abi16_object(struct nouveau_object *, int (**)(struct nouveau_object *));
+drm_private void abi16_delete(struct nouveau_object *);
+drm_private int  abi16_sclass(struct nouveau_object *, struct nouveau_sclass **);
+drm_private void abi16_bo_info(struct nouveau_bo *, struct drm_nouveau_gem_info *);
+drm_private int  abi16_bo_init(struct nouveau_bo *, uint32_t alignment,
+			       union nouveau_bo_config *);
 
 #endif
